<?php
/**
 * Copyright since 2007 PrestaShop SA and Contributors
 * PrestaShop is an International Registered Trademark & Property of PrestaShop SA.
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License version 3.0
 * that is bundled with this package in the file LICENSE.md.
 * It is also available through the world-wide-web at this URL:
 * https://opensource.org/licenses/AFL-3.0
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * @author    PrestaShop SA and Contributors <contact@prestashop.com>
 * @copyright Since 2007 PrestaShop SA and Contributors
 * @license   https://opensource.org/licenses/AFL-3.0 Academic Free License version 3.0
 */
function f24Options()
{
    $f24_fields = [
        'PS_FATTURA24_API',
        'PS_SALVA_CLIENTE',
        'PS_CREA_ORDINE',
        'PS_ABILITA_ORDINE_ZERO',
        'PS_PDF_ORDINE_',
        'PS_MAIL_ORDINE',
        'PS_F24_MODELLO_ORDINE',
        'PS_F24_MODELLO_ORDINE_DEST',
        'PS_CREAZIONE_FATTURA',
        'PS_ABILITA_FATTURA_ZERO',
        'PS_CONF_NATURA_IVA',
        'PS_CONF_NATURA_SPEDIZIONE',
        'PS_INV_OBJECT',
        'PS_EMAIL_FATTURA',
        'PS_STATO_PAGATO',
        'PS_DISABILITA_RICEVUTE',
        'PS_F24_MODELLO_FATTURA',
        'PS_F24_MODELLO_FATTURA_DEST',
        'PS_F24_PDC',
        'PS_F24_SEZIONALE_RICEVUTA',
        'PS_F24_SEZIONALE_FATTURA',
        'PS_F24_BOLLO_VIRTUALE_FE',
    ];

    return $f24_fields;
}

function f24Value($param)
{
    return [$param => Tools::getValue($param), Configuration::get($param)];
}

function getConfigFieldsValues()
{
    $optionKeys = [
        'PS_FATTURA24_API',
        'PS_SALVA_CLIENTE',
        'PS_CREA_ORDINE_',
        'PS_ABILITA_ORDINE_ZERO',
        'PS_PDF_ORDINE_',
        'PS_MAIL_ORDINE_',
        'PS_F24_MODELLO_ORDINE_',
        'PS_F24_MODELLO_ORDINE_DEST_',
        'PS_CREAZIONE_FATTURA_',
        'PS_ABILITA_FATTURA_ZERO',
        'PS_CONF_NATURA_IVA',
        'PS_CONF_NATURA_SPEDIZIONE',
        'PS_INV_OBJECT',
        'PS_EMAIL_FATTURA_',
        'PS_STATO_PAGATO_',
        'PS_DISABILITA_RICEVUTE_',
        'PS_F24_MAGAZZINO_ORDINE_',
        'PS_F24_MAGAZZINO_FATTURA_',
        'PS_F24_MODELLO_FATTURA_',
        'PS_F24_MODELLO_FATTURA_DEST_',
        'PS_F24_PDC_',
        'PS_F24_SEZIONALE_FATTURA_',
        'PS_F24_SEZIONALE_RICEVUTA_',
        'PS_F24_BOLLO_VIRTUALE_FE',
    ];
    $result = [];
    foreach ($optionKeys as $key) {
        $result[$key] = Configuration::get($key);
    }

    return $result;
}
function getListaNatureNew()
{
    $tipiAliquoteIva = [
        'N0' => 'Nessuna',
        'N1' => ['N1' => 'N1 - Escluse ex art. 15'],
        'N2' => ['N2.1' => 'N2.1 - Non soggette ad IVA ai sensi degli artt. da 7 a 7-septies del DPR 633/72',
                 'N2.2' => 'N2.2 - Altri casi', ],
        'N3' => ['N3.1' => 'N3.1 - Esportazioni',
                 'N3.2' => 'N3.2 - Cessioni intracomunitarie',
                 'N3.3' => 'N3.3 - Cessioni verso San Marino',
                 'N3.4' => 'N3.4 - Operazioni assimilate alle cessioni all\'esportazione',
                 'N3.5' => 'N3.5 - A seguito di dichiarazioni d\'intento',
                 'N3.6' => 'N3.6 - Altre operazioni che non concorrono alla formazione del plafond', ],
        'N4' => ['N4' => 'N4 - Esenti'],
        'N5' => ['N5' => 'N5 - Regime del margine / IVA non esposta in fattura'],
        'N6' => ['N6.1' => 'N6.1 - Cessione di rottami e altri materiali di recupero',
                 'N6.2' => 'N6.2 - Cessione di oro e argento puro',
                 'N6.3' => 'N6.3 - Subappalto nel settore edile',
                 'N6.4' => 'N6.4 - Cessione di fabbricati',
                 'N6.5' => 'N6.5 - Cessione di telefoni cellulari',
                 'N6.6' => 'N6.6 - Cessione di prodotti elettronici',
                 'N6.7' => 'N6.7 - Prestazioni comparto edile e settori connessi',
                 'N6.8' => 'N6.8 - Operazioni settore energetico',
                 'N6.9' => 'N6.9 - Altri casi', ],
        'N7' => ['N7' => 'N7 - IVA assolta in altro stato UE'],
        ];

    return $tipiAliquoteIva;
}
function updateConfig()
{
    Configuration::updateValue('PS_FATTURA24_API', $_REQUEST['PS_FATTURA24_API']);

    if (isset($_REQUEST['PS_CONF_NATURA_IVA'])) {
        Configuration::updateValue('PS_CONF_NATURA_IVA', $_REQUEST['PS_CONF_NATURA_IVA']);
    }
    if (isset($_REQUEST['PS_CONF_NATURA_SPEDIZIONE'])) {
        Configuration::updateValue('PS_CONF_NATURA_SPEDIZIONE', $_REQUEST['PS_CONF_NATURA_SPEDIZIONE']);
    }
    if (isset($_REQUEST['PS_INV_OBJECT'])) {
        Configuration::updateValue('PS_INV_OBJECT', $_REQUEST['PS_INV_OBJECT']); // oggetto del documento
    }
    // se le checkbox sono settate imposto il valore della relativa configurazione a 1, altrimenti a 0
    if (isset($_REQUEST['PS_SALVA_CLIENTE_'])) {
        Configuration::updateValue('PS_SALVA_CLIENTE', $_REQUEST['PS_SALVA_CLIENTE_']);
    } else {
        Configuration::updateValue('PS_SALVA_CLIENTE', '0');
    }

    if (isset($_REQUEST['PS_CREA_ORDINE_'])) {
        Configuration::updateValue('PS_CREA_ORDINE', $_REQUEST['PS_CREA_ORDINE_']);
    } else {
        Configuration::updateValue('PS_CREA_ORDINE', '0');
    }

    if (isset($_REQUEST['PS_ABILITA_ORDINE_ZERO'])) {
        Configuration::updateValue('PS_ABILITA_ORDINE_ZERO', $_REQUEST['PS_ABILITA_ORDINE_ZERO']);
    } else {
        Configuration::updateValue('PS_ABILITA_ORDINE_ZERO', '0');
    }

    if (isset($_REQUEST['PS_NUMERO_ORDINE'])) {
        Configuration::updateValue('PS_NUMERO_ORDINE', $_REQUEST['PS_NUMERO_ORDINE']);
    } else {
        Configuration::updateValue('PS_NUMERO_ORDINE', '0');
    }

    if (isset($_REQUEST['PS_PDF_ORDINE_'])) {
        Configuration::updateValue('PS_PDF_ORDINE', $_REQUEST['PS_PDF_ORDINE_']);
    } else {
        Configuration::updateValue('PS_PDF_ORDINE', '0');
    }

    if (isset($_REQUEST['PS_MAIL_ORDINE_'])) {
        Configuration::updateValue('PS_MAIL_ORDINE', $_REQUEST['PS_MAIL_ORDINE_']);
    } else {
        Configuration::updateValue('PS_MAIL_ORDINE', '0');
    }

    if (isset($_REQUEST['PS_F24_MODELLO_ORDINE_'])) {
        Configuration::updateValue('PS_F24_MODELLO_ORDINE', $_REQUEST['PS_F24_MODELLO_ORDINE_']);
    } else {
        Configuration::updateValue('PS_F24_MODELLO_ORDINE', 'Default');
    }

    if (isset($_REQUEST['PS_F24_MODELLO_ORDINE_DEST_'])) {
        Configuration::updateValue('PS_F24_MODELLO_ORDINE_DEST', $_REQUEST['PS_F24_MODELLO_ORDINE_DEST_']);
    } else {
        Configuration::updateValue('PS_F24_MODELLO_ORDINE_DEST', 'Default');
    }

    if (isset($_REQUEST['PS_CREAZIONE_FATTURA_'])) {
        Configuration::updateValue('PS_CREAZIONE_FATTURA', $_REQUEST['PS_CREAZIONE_FATTURA_']);
    } else {
        Configuration::updateValue('PS_CREAZIONE_FATTURA', '0');
    }

    if (isset($_REQUEST['PS_ABILITA_FATTURA_ZERO'])) {
        Configuration::updateValue('PS_ABILITA_FATTURA_ZERO', $_REQUEST['PS_ABILITA_FATTURA_ZERO']);
    } else {
        Configuration::updateValue('PS_ABILITA_FATTURA_ZERO', '0');
    }

    if (isset($_REQUEST['PS_EMAIL_FATTURA_'])) {
        Configuration::updateValue('PS_EMAIL_FATTURA', $_REQUEST['PS_EMAIL_FATTURA_']);
    } else {
        Configuration::updateValue('PS_EMAIL_FATTURA', '0');
    }

    if (isset($_REQUEST['PS_STATO_PAGATO_'])) {
        Configuration::updateValue('PS_STATO_PAGATO', $_REQUEST['PS_STATO_PAGATO_']);
    } else {
        Configuration::updateValue('PS_STATO_PAGATO', '0');
    }

    if (isset($_REQUEST['PS_DISABILITA_RICEVUTE_'])) {
        Configuration::updateValue('PS_DISABILITA_RICEVUTE', $_REQUEST['PS_DISABILITA_RICEVUTE_']);
    } else {
        Configuration::updateValue('PS_DISABILITA_RICEVUTE', '0');
    }

    if (isset($_REQUEST['PS_F24_MODELLO_FATTURA_'])) {
        Configuration::updateValue('PS_F24_MODELLO_FATTURA', $_REQUEST['PS_F24_MODELLO_FATTURA_']);
    } else {
        Configuration::updateValue('PS_F24_MODELLO_FATTURA', 'Default');
    }

    if (isset($_REQUEST['PS_F24_MODELLO_FATTURA_DEST_'])) {
        Configuration::updateValue('PS_F24_MODELLO_FATTURA_DEST', $_REQUEST['PS_F24_MODELLO_FATTURA_DEST_']);
    } else {
        Configuration::updateValue('PS_F24_MODELLO_FATTURA_DEST', 'Default');
    }

    if (isset($_REQUEST['PS_F24_PDC_'])) {
        Configuration::updateValue('PS_F24_PDC', $_REQUEST['PS_F24_PDC_']);
    } else {
        Configuration::updateValue('PS_F24_PDC', 'Default');
    }

    if (isset($_REQUEST['PS_F24_SEZIONALE_RICEVUTA_'])) {
        Configuration::updateValue('PS_F24_SEZIONALE_RICEVUTA', $_REQUEST['PS_F24_SEZIONALE_RICEVUTA_']);
    } else {
        Configuration::updateValue('PS_F24_SEZIONALE_RICEVUTA', 'Default');
    }

    if (isset($_REQUEST['PS_F24_SEZIONALE_FATTURA_'])) {
        Configuration::updateValue('PS_F24_SEZIONALE_FATTURA', $_REQUEST['PS_F24_SEZIONALE_FATTURA_']);
    } else {
        Configuration::updateValue('PS_F24_SEZIONALE_FATTURA', 'Default');
    }
    if (isset($_REQUEST['PS_F24_BOLLO_VIRTUALE_FE'])) {
        Configuration::updateValue('PS_F24_BOLLO_VIRTUALE_FE', $_REQUEST['PS_F24_BOLLO_VIRTUALE_FE']);
    }
}
